function [Image, Mc, Out] = PDFT1D (In, IP, prior, regul, mode, dispecho)
% function [Image, Mc, Out] = PDFT2D (In, IP, prior, pprior, regul, mode, dispecho) 
% Spectral estimation of 2D images via PDFT
% 
% Parameters:
% In:       Matrix containing input data
%           column 1: spatial frequencies
%           column 2: measured data (complex values)
%
% IP:       3 x 1 vector; parameters specifying image:
%           row 1: no. of image pixels
%           row 2: sampling distance
%           row 3: shift of image window
%
% prior:    n x 4 matrix specifying the prior;
%           the total prior is defined as the superposition of n sub-strutures
%           each sub-structures is defined by 5 entries:
%           column 1:    number key specifying type of prior (<300 matlab; >300 C code); 
%                        implemented are:
%                        rectangle -> function 'rect1D'; key: 201/301; 2 additional parameters
%           column 2:    weight (scalar)
%           column 3-4:  additional parameters to specify prior; for prior specified with less than
%                        four parameters, the matrix must be filled with dummy values.
%
% mode:     specifies subroutines called
%           'matlab' (default): calls subroutine coded exclusively in Matlab
%           'ccode'           : calls subroutines coded in c (only available for Windows)
%
% dispecho: specfifies echo output mode
%           'show' (default)  : outputs progress information
%           'quiet'           : runs without output
%
% Image:    image estimate
% Mc:       P-matrix
% Out:      spatial frequencies and PDFT coefficients (same format as In)
%
% see also: 'Rect1D', 'ftRect1D', 'pMeval', 'mPeval', 
%
% (c) Markus Testorf, Vers. 6.0, November 22, 2002
%

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

i=sqrt(-1.0);        % .... because Matlab has its downsides

if nargin < 5,
   mode     = 'matlab';
   dipsecho = 'show'
elseif nargin < 6,
    dispecho = 'show';
end

if strcmp(dispecho, 'show'), disp('- PDFT1D algorithm, (c) M. Testorf, Vers. 1.0, Nov. 22, 2002 -'); end

% setup system of equations
if strcmp(dispecho, 'show'), disp('PDFT1D: setup system of linear equations ...'); end
Mc = genPmatrix1D (In(:,1), prior, regul);

% solve system of linear equations
if strcmp(dispecho, 'show'), disp('PDFT1D: solve system of linear equations ...'); end
xk = Mc\In(:,2);
In(:,2) = xk;
clear xk;

% calculate image estimate via DFT
if strcmp(dispecho, 'show'), disp('PDFT1D: compute discrete Fourier series ...'); end
Image = idft1D (In, IP, mode, 'quiet');

if strcmp(dispecho, 'show'), disp('PDFT1D: multiply with prior ...'); end
Image = Image .* CalcPrior1D (IP, prior);


Out = In;
