function [Image, Mc, Out] = PDFT2D (In,IP,prior, regul)
% function [Image, Mc, Out] = PDFT2D (In, IP, prior, regul, mode, dispecho) 
% Spectral estimation of 2D images via PDFT
% 
% Parameters:
% In:       Matrix containing input data
%           column 1: nu  spatial frequency in x
%           column 2: mu  spatial frequency in y
%           column 3: measured data (complex values)
%
% IP:       3 x 2 matrix; parameters specifying image:
%           row 1: vector containing no. of image pixels in x and y
%           row 2: sampling distance of image in x an y
%           row 3: shift of image window in x an y
%
% prior:    n x 6 matrix specifying the prior;
%           the total prior is defined as the superposition of n sub-strutures
%           each sub-structures is defined by 5 entries:
%           column 1:    number key specifying type of prior (<100 matlab; >100 C code); 
%                        implemented are:
%                        rectangle -> function 'rect2D'; key: 1/101; 4 additional parameters
%			             disk      -> function 'disk2D'; key: 2/102; 3 additional parameters
%           column 2:    weight (scalar)
%           column 3-6:  additional parameters to specify prior; for prior specifued with less than
%                          four parameters, the matrix must be filled with dummy values.
%
% mode:     specifies subroutines called
%           'matlab' (default): calls subroutine coded exclusively in Matlab
%           'ccode'           : calls subroutines coded in c (only available for Windows)
%
% dispecho: specfifies echo output mode
%           'show' (default)  : outputs progress information
%           'quiet'           : runs without output
%
% Image:    image estimate
% Mc:       P-matrix
% Out:      spatial frequencies and PDFT coefficients (same format as In)
%
% see also: 'Rect2D', 'Disk2D', 'ftRect2D', 'ftDisk2D', 'pMeval', 'mPeval', 
%
% (c) Markus Testorf, Vers. 6.0, November 22, 2002
%

% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

i=sqrt(-1.0);        % .... because Matlab has its downsides

%if nargin < 5,
%   mode     = 'matlab';
%   dipsecho = 'show'
%elseif nargin < 6,
%    dispecho = 'show';
%end

%if strcmp(dispecho, 'show'), disp('- PDFT2D algorithm, (c) M. Testorf, Vers. 6.0, Nov. 22, 2002 -'); end

% setup system of equations
%if strcmp(dispecho, 'show'), 
fprintf('\n\n');
fprintf('Setting up system of linear equations for PDFT ...');

Mc = genPmatrix2D(In, prior, regul);

% solve system of linear equations
%if strcmp(dispecho, 'show'), 
fprintf('\n\n');
disp('Solving system of linear equations for PDFT ...');
xk = Mc\In(:,3);
In(:,3) = xk;
clear xk;

% calculate image estimate via DFT
%if strcmp(dispecho, 'show'), 
fprintf('\n\n');
disp('Computing discrete Fourier series for PDFT ...');
Image = idft2D(In, IP);

%if strcmp(dispecho, 'show'), 
%    disp('PDFT2D: multiply with prior ...');
Image = Image .* calcPrior2D(IP, prior);

Out = In;
